function require(p){
  var path = require.resolve(p)
    , mod = require.modules[path];
  if (!mod) throw new Error('failed to require "' + p + '"');
  if (!mod.exports) {
    mod.exports = {};
    mod.call(mod.exports, mod, mod.exports, require.relative(path));
  }
  return mod.exports;
}

require.modules = {};

require.resolve = function (path){
  var orig = path
    , reg = path + '.js'
    , index = path + '/index.js';
  return require.modules[reg] && reg
    || require.modules[index] && index
    || orig;
};

require.register = function (path, fn){
  require.modules[path] = fn;
};

require.relative = function (parent) {
  return function(p){
    if ('.' != p[0]) return require(p);

    var path = parent.split('/')
      , segs = p.split('/');
    path.pop();

    for (var i = 0; i < segs.length; i++) {
      var seg = segs[i];
      if ('..' == seg) path.pop();
      else if ('.' != seg) path.push(seg);
    }

    return require(path.join('/'));
  };
};

require.register("config", function(module, exports, require){
module.exports = {"host":"soccer.stagas.com", "port":"7054"}
});
require.register("util", function(module, exports, require){
module.exports = {"inherits":function (ctor, superCtor) {
  ctor.super_ = superCtor;
  ctor.prototype = Object.create(superCtor.prototype, {
    constructor: {
      value: ctor,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
}}
});
require.register("./globals", function(module, exports, require){
module.exports = {}

});
require.register("./utils", function(module, exports, require){
Math.HalfPI = Math.PI / 180

Math.cosa = function (a) {
  return Math.round(Math.cos(a * Math.HalfPI) * 10000) / 10000
}

Math.sina = function (a) {
  return Math.round(Math.sin(a * Math.HalfPI) * 10000) / 10000
}

Math.sgn = function (a) {
  if (a<0) return -1
  else if (a>0) return 1
  else return 0
}

exports.clone = function clone (obj) {
  if(obj == null || typeof(obj) != 'object')
    return obj

  var temp = obj.constructor()

  for(var key in obj)
    temp[key] = clone(obj[key])
  return temp
}

//+ Jonas Raoni Soares Silva
//@ http://jsfromhell.com/math/is-point-in-poly [rev. #0]

exports.isPointInPoly = function isPointInPoly (poly, pt){
    for(var c = false, i = -1, l = poly.length, j = l - 1; ++i < l; j = i)
        ((poly[i].y <= pt.y && pt.y < poly[j].y) || (poly[j].y <= pt.y && pt.y < poly[i].y))
        && (pt.x < (poly[j].x - poly[i].x) * (pt.y - poly[i].y) / (poly[j].y - poly[i].y) + poly[i].x)
        && (c = !c);
    return c;
}

//+ Jonas Raoni Soares Silva
//@ http://jsfromhell.com/math/dot-line-length [rev. #1]

exports.dotLineLength = function dotLineLength (x, y, x0, y0, x1, y1, o){
    function lineLength(x, y, x0, y0){
        return Math.sqrt((x -= x0) * x + (y -= y0) * y);
    }
    if(o && !(o = function(x, y, x0, y0, x1, y1){
        if(!(x1 - x0)) return {x: x0, y: y};
        else if(!(y1 - y0)) return {x: x, y: y0};
        var left, tg = -1 / ((y1 - y0) / (x1 - x0));
        return {x: left = (x1 * (x * tg - y + y0) + x0 * (x * - tg + y - y1)) / (tg * (x1 - x0) + y0 - y1), y: tg * left - tg * x + y};
    }(x, y, x0, y0, x1, y1), o.x >= Math.min(x0, x1) && o.x <= Math.max(x0, x1) && o.y >= Math.min(y0, y1) && o.y <= Math.max(y0, y1))){
        var l1 = lineLength(x, y, x0, y0), l2 = lineLength(x, y, x1, y1);
        return l1 > l2 ? l2 : l1;
    }
    else {
        var a = y0 - y1, b = x1 - x0, c = x0 * y1 - y0 * x1;
        return Math.abs(a * x + b * y + c) / Math.sqrt(a * a + b * b);
    }
}

exports.getRandom = function getRandom(arr) {
  return arr[Math.floor(Math.random() * arr.length)]
}

exports.getRandomProperty = function getRandomProperty(obj) {
  var result
    , count = 0
  for (var prop in obj)
    if (Math.random() < 1/++count)
      result = prop
  return result
}

exports.rotate = function rotate (a, cp, ar) {
  var newar = []
  
  for (var i=0; i<ar.length; i+=2) {
    newar.push(
      { x: ((cp.x - ar[i]) * Math.cosa(a) - (cp.y - ar[i+1]) * Math.sina(a)) + cp.x 
      , y: ((cp.x - ar[i]) * Math.sina(a) + (cp.y - ar[i+1]) * Math.cosa(a)) + cp.y 
      })
  }
  
  return newar
}

exports.pick = function pick (obj, keyArray) {
  var ret = {}
  if ('string' === typeof keyArray)
    keyArray = [].slice.call(arguments, 1)
  keyArray.forEach(function (key) {
    ret[key] = obj[key] || 0
  })
  return ret
}

});
require.register("./state", function(module, exports, require){
/* 
 *
 *  State
 *
 */

var utils = require('./utils')
var Timer = require('./timer')

function State (game) {
  this.game = game

  // current and previous state arrays
  this.current = []
  this.previous = []

  this.watched = []

  this.currentDelta = []
  this.previousDelta = []

  this.subList = {}
  this.pubList = {}

  this.alias = {}

  this.timer = new Timer()
}

State.prototype = {
  clear: function() {
    this.current = []
    return this
  }

, remember: function() {
    // remember current state
    this.previous = this.current.slice(0)
    return this
  }

, add: function (obj, state) {
    this.current.push({ obj: obj, state: state })
  }

, update: function () {
    var self = this
    this.watched.forEach(function (obj) {
      if (obj.update) {
        obj.update(self.game)
        self.add(obj, utils.pick(obj, 'x', 'y', 'z'))
      }
    })
  }

, tick: function (ms) {
    this.timer.tick()
    while (this.timer.overflow(ms)) {
      this.remember().clear()
      this.update()
    }
    var delta = this.delta()

    if (delta.length) {}

    this.render(this.alpha(this.timer.alpha(ms)))
  }

, watch: function (obj) {
    // add a new frame to current state  
    this.watched.push(obj)
  }

  // calculate alpha state position
, alpha: function (a) {
    // we need two to make an alpha
    if (this.current.length === 0 || this.previous.length === 0)
      return []

    // alpha state array
    var aState = []

    // go through the previous state
    for (var i = 0, l = this.previous.length; i < l; i++) {

      // alpha frame object
      var aFrame = {}

      // do we have everything?
      if (this.current && this.current[i] && this.current[i].state
        && this.previous && this.previous[i] && this.previous[i].state) {
        // go through the previous state properties
        for (var p in this.previous[i].state) {
          switch (typeof this.previous[i].state[p]) {
            case 'number':
              // linear interpolation with current state properties
              aFrame[p] = ( this.current[i].state[p] * a )
                        + ( this.previous[i].state[p] * (1 - a) )                 
              break
            default:
              if (a >= 0.5) aFrame[p] = this.current[i].state[p]
              else aFrame[p] = this.previous[i].state[p]
              break
          }  
        }
      }

      // if we're ok, push alpha frame to alpha state
      if (this.current && this.current[i]) {
        aState.push({ obj: this.current[i].obj, state: aFrame })
      }
    }

    return aState
  }

, delta: function () {
    if (this.previousDelta.length === 0) {
      this.previousDelta = this.current.slice(0)
      return []
    }

    // delta state array
    var dState = this.current.slice(0)

    // publish state array
    var pState = []

    var d = {}

    for (var i=0, l=this.previousDelta.length; i<l; i++) {
      var pFrame = {}
      var changed = false

      if (typeof this.subList[this.previousDelta[i].obj.name] !== 'undefined') {
        for (var p in this.previousDelta[i].state) {
          if (typeof this.subList[this.previousDelta[i].obj.name][p] !== 'undefined') {
            if (dState[i].state[p] != this.previousDelta[i].state[p]) {
              switch (typeof dState[i].state[p]) {
              case 'number':
                if (dState[i].state[p].toFixed(2) != this.previousDelta[i].state[p].toFixed(2)) {
                  pFrame[p] = (dState[i].state[p] - this.previousDelta[i].state[p]).toFixed(2)
                  changed[p] = true
                }
                break
              default:
                pFrame[p] = dState[i].state[p]
                changed[p] = true
                break
              }
            }
          }
        }
      }

      pState.push({obj: this.current[i].obj, state: pFrame})
    }

    this.previousDelta = dState.slice(0)
    this.pubState = dState.slice(0)
    return dState
  }

  // render a state array
, render: function (rState) {
    for (var i=0, l=rState.length; i<l; i++) {
      if (rState[i].obj.display) rState[i].obj.display(rState[i].state)
    }
  }
}

module.exports = State

});
require.register("./timer", function(module, exports, require){
/*
 *
 *  Timer
 *
 */
 
function Timer () {
  this.name = 'Timer'
  
  this.now = new Date()
  this.before = this.now
  this.delta = 0
  this.accumulator = 0
}

Timer.prototype = {
  // calculate new tick
  tick: function() {
    this.now = new Date()
    this.delta = this.now-this.before
    this.accumulator += this.delta
    this.before = this.now
  }

  // overflow timer based on given ms
, overflow: function (ms) {
    if (this.accumulator >= ms) {
      this.accumulator -= ms
      return true
    }
    return false
  }

  // calculate alpha
, alpha: function (ms) {
    return (this.accumulator / ms)
  }
}

module.exports = Timer

});
require.register("./camera", function(module, exports, require){
/*
 *
 *  Camera
 *
 */

var scroll = window.scrollTo || window.scroll

function Camera (props) {
  this.name = 'Camera'

  // bounce float (0..1 : larger = more bounce)
  this.bounce = 0.93
  
  // scaling (smaller = faster)
  this.scale = 400

  // xy coords
  this.x = 0
  this.y = 0

  // xy velocity
  this.vx = 0
  this.vy = 0

  // xy delta to target
  this.dx = 0
  this.dy = 0

  // property override
  for (var p in props) {
    this[p] = props[p]
  }
}

Camera.prototype.update = function () {
  // calculate velocity
  this.vx = this.bounce * this.vx - (this.x - this.dx) / this.scale
  this.vy = this.bounce * this.vy - (this.y - this.dy) / this.scale
  
  // apply to coords but not if it's too small (so it doesn't flicker when being ~still)
  this.x += Math.abs(this.vx) < 0.3 ? 0 : this.vx
  this.y += Math.abs(this.vy) < 0.3 ? 0 : this.vy
}

Camera.prototype.display = function (v) {
  // scroll window to alpha state
  scroll(v.x, v.y)
}
  
Camera.prototype.moveTo = function (coords) {
  // set delta to the center of our viewport
  this.dx = coords.x - (exports.windowWidth / 2)
  this.dy = coords.y - (exports.windowHeight / 2)
}

module.exports = Camera

exports.windowWidth = window.innerWidth
exports.windowHeight = window.innerHeight

function onresize () {
  exports.windowWidth = window.innerWidth
  exports.windowHeight = window.innerHeight
}
window.addEventListener('resize', onresize)

});
require.register("./stadium", function(module, exports, require){
/*
 *
 *  Stadium
 *
 */

function Stadium (props) {
  this.bounds = {
    left: 102
  , top: 160
  , right: 1250
  , bottom: 1570 
  }

  this.field = {
    left: 162
  , top: 226
  , right: 1183
  , bottom: 1500
  }
  
  this.keeper = {
    top: {
      bottom: this.field.top + 176
    }
  , bottom: {
      top: this.field.bottom - 176
    }
  , left: this.field.left + 226
  , right: this.field.right - 226
  }

  this.post = { 
    left: 602
  , right: 743
  , top: 228
  , bottom: 1506
  , z: 38
  , depthTop: 28
  , depthBottom: 23
  }
  
  this.posts = {
    vert: [
      { x: this.post.left, y: this.post.top }
    , { x: this.post.right, y: this.post.top }
    , { x: this.post.left, y: this.post.bottom }
    , { x: this.post.right, y: this.post.bottom }
    ]
    
  , horiz:
    [
      { y: this.post.top, z: this.post.z }
    , { y: this.post.bottom, z: this.post.z }
    ]
  }
}

Stadium.prototype.init = function () {
  var $stadium = document.getElementById('stadium')
  var tsx=336
  var tsy=1696
  var tn=4
  var tm=4
  var t=[]
  var tx=0
  var ty=0
  for (var i=0; i<tm; i++) {
    t.push('<div style="')
    t.push('background:url(swpt_' + i + '.png) top left no-repeat; height:'+tsy+'px; width:'+tsx+'px; float:left; ')
    t.push('position:absolute; top:' + (ty*tsy) + 'px; left:' + (tx) + 'px; z-index:1; ')
    tx+=tsx
    if (tx==tn*tsx) {
      ty++
      tx=0
    }
    t.push('"></div>')
  }
  $stadium.style.width = (tsx*tn) + 'px'
  $stadium.style.height = ((tm/tn)*tsy) + 'px'
  $stadium.innerHTML = t.join('')
}

module.exports = Stadium

});
require.register("./keys", function(module, exports, require){
function Keys () {
  this.keys = 0
  this.key = { left: 0, up: 0, right: 0, down: 0, shoot: 0 }
  this.shootkey = [ 13,96,17,32,18,16,190,191,90,122,88,120 ]
  this.shootThreshold = 120
  this.shootTimer = 0
  this.shootTimeout = 0
}

Keys.prototype = {  
  init: function(game) {
    var self = this
    this.game = game
    document.onkeydown = function(e) {
      return self.checkkeydown(e)
    }
    document.onkeyup   = function(e) {
      return self.checkkeyup(e)
    }
  }
  
, checkkeydown: function(e) {
    var self = this
    var chr
    if (typeof e.which === 'undefined') chr = e.keyCode    // IE
    else if (typeof e.which !== 'undefined' && typeof e.charCode !== 'undefined') chr = e.which   // All others
    //console.log(chr)
    switch (chr) {
      case 37:
        this.key.right=0
        this.key.left=1
        break
      case 38:
        this.key.down=0
        this.key.up=2
        break
      case 39:
        this.key.left=0
        this.key.right=4
        break
      case 40:
        this.key.up=0
        this.key.down=8
        break
      case 220:
      case 221:
      case 34:
        master.switchLooks()
        break
      case 77:
        master.switchMonkey()
        break
    }
    if (~this.shootkey.indexOf(chr)) {
      if (!this.key.shoot && this.game.master.checkBallOwner(this.game.ball)) {
        //master.kicks=true
        
        this.shootTimer = new Date()

        this.shootTimeout = setTimeout(function() {
          if (self.key.shoot && self.game.master.checkBallOwner(self.game.ball)) {
            self.game.ball.shoot()
          }
        }, this.shootThreshold)
      }
      this.key.shoot = 1
    }
    this.keys
    = this.key.left
      + this.key.up
      + this.key.right
      + this.key.down //+key.shoot

    return false
  }
  
, checkkeyup: function(e) {
    var chr
    if (typeof e.which === 'undefined') chr = e.keyCode    // IE
    else if (typeof e.which !== 'undefined' && typeof e.charCode !== 'undefined') chr = e.which   // All others

    switch (chr) {
      case 37:
        this.key.left=0
        break
      case 38:
        this.key.up=0
        break
      case 39:
        this.key.right=0
        break
      case 40:
        this.key.down=0
        break
    }
    if (~this.shootkey.indexOf(chr)) {
      this.key.shoot = 0
      var dateNow = new Date()
      if (dateNow-this.shootTimer < this.shootThreshold) {
        clearTimeout(this.shootTimeout)
        if (this.game.master.checkBallOwner(this.game.ball)) {
          //master.kicks=true
          this.game.ball.pass()
        }
      }
    }
    this.keys = this.key.left+this.key.up+this.key.right+this.key.down //+key.shoot
    return false
  }

, getKeys: function() {
    return this.keys
  }
}

module.exports = Keys

});
require.register("./keeper", function(module, exports, require){
var utils = require('./utils')
var Sprite = require('./sprite')
var Stadium = require('./stadium')
var stadium = new Stadium()

var combinations = [
  'white_black'
, 'white_brown'
, 'white_blonde'
, 'black_black'
, 'black_brown'
, 'black_blonde'
]

// Direction in radius
var Direction = {
  'up': 270
, 'down': 90
, 'left': 180
, 'right': 0
, 'left + up': 225
, 'right + up': 315
, 'left + down': 135
, 'right + down': 45
, '' : null
}

// Direction by keys
var KeysDirection = {
  0: '' // still
, 1: 'left' //Direction.left
, 2: 'up' //Direction.up
, 3: 'left + up' //Direction.leftUp
, 4: 'right' //Direction.right
, 5: 'left + right' // left + right
, 6: 'right + up' //Direction.rightUp
, 7: '' // impossible
, 8: 'down' //Direction.down
, 9: 'left + down' //Direction.leftDown // left + down
, 10: 'up + down' //null // up + down
, 12: 'right + down' //Direction.rightDown
}

function Keeper (game, number, uid, home_away) {
  this.game = game

  this.number = number

  this.uid = uid || Math.floor(Math.random() * 100)

  this.home_away = home_away

  this.initDirection = home_away === 'home' ? 'up' : 'down'

  // actual sprite position
  this.x = 0
  this.y = 0

  // walk to xy
  this.wtx = this.x
  this.wty = this.y

  // player feet position
  this.cx = 0
  this.cy = 0

  this.keys = 0

  this.feet = []

  this.vx = 0
  this.vy = 0

  this.a = 0

  this.ox = this.x
  this.oy = this.y

  this.monkey = false
  this.monkeyDate = new Date()

  this.walks = false

  this.runSpeed = 3.6
  this.dribbleSpeed = 3.4

  this.speed = this.runSpeed

  this.magnet = 0.1
  this.direction = ''
  this.olddirection = ''
  this.kicks = false

  this.kickPower = 12
  this.passPower = 8

  this.jumping = 0
  this.isKeeper = true

  //var self = this
  /*setInterval(function () {
    self.jump(utils.getRandom(['right', 'left']))
  }, 5000) */
  this.team = {}

  this.cpu = false

  return this
}

Keeper.prototype = {
  createSprite: function () {
    this.sprite = new Sprite(
      'keeper_' + this.uid
      , 'keeper ' + this.home_away + ' ' + this.looks
      , [32, 40]
      , { 'up' : [
            [0,10,4,2]
          , [1,10,4,0]
          , [0,10,4,2]
          , [2,10,4,4]
          ]
        , 'down' : [
            [3,10,4,2]
          , [4,10,4,4]
          , [3,10,4,2]
          , [5,10,4,0]
          ]
        , 'right' : [
            [6,10,4,6]
          , [7,10,4,2]
          , [6,10,4,6]
          , [8,10,4,4]
          ]
        , 'left' : [
            [9,10,4,6]
          , [10,10,4,4]
          , [9,10,4,6]
          , [11,10,4,4]
          ]
        , 'left + down' : [
            [13,10,4,8]
          , [12,10,4,6]
          , [13,10,4,8]
          , [14,10,4,4]
          ]
        , 'right + down' : [
            [16,10,4,4]
          , [15,10,4,4]
          , [16,10,4,4]
          , [17,10,4,2]
          ]
        , 'left + up' : [
            [19,10,4,8]
          , [18,10,4,4]
          , [19,10,4,8]
          , [20,10,4,6]
          ]
        , 'right + up' : [
            [22,10,4,4]
          , [21,10,4,2]
          , [22,10,4,4]
          , [23,10,4,4]
          ]
        , 'jump down + right': [
            [24,5,-2,4,2,-4]
          , [25,5,-4,4,2,-4]
          , [26,5,-4,4,2,-4]
          , [27,5,-2,4,4,-4]
          , [28,5,6,4,6,-4]
          , [29,1000,8,4,6,-4]          
          ]
        , 'jump down + left': [
            [36,5,-4,4,4,-4]
          , [35,5,-2,4,2,-4]
          , [34,5,-2,4,2,-4]
          , [33,5,-2,4,4,-4]
          , [32,5,6,4,6,-4]
          , [31,1000,8,4,6,-4]          
          ]
        , 'jump up + right': [
            [38,5,-2,4,2,-4]
          , [39,5,-4,4,2,-4]
          , [40,5,-4,4,2,-4]
          , [41,5,-2,4,4,-4]
          , [42,5,6,4,6,-4]
          , [43,1000,8,4,6,-4]
          ]
        , 'jump up + left': [
            [50,5,-4,4,4,-4]
          , [49,5,-2,4,2,-4]
          , [48,5,-2,4,2,-4]
          , [47,5,-2,4,4,-4]
          , [46,5,6,4,6,-4]
          , [45,1000,8,4,6,-4]
          ]          
        }
      )

    this.makeFeetPlanes()
    this.sprite.face = utils.getRandomProperty(this.sprite.map)
  }

, handleKeys: function () {
    this.olddirection = this.direction

    this.ox = this.x
    this.oy = this.y

    if (this.isMaster) {
      this.keys = keys.getKeys()
      this.turnTo(KeysDirection[this.keys])
      if (this.direction === '') this.walks = false
      else this.walks = true
    }
    else {
      this.turnTo(KeysDirection[this.keys])
      if (this.direction === '') this.walks = false
      else this.walks = true
    }
  }

, setLooks: function (looks) {
    this.looks = looks
  }

, randomLooks: function () {
    this.setLooks(utils.getRandom(combinations))
  }

, switchLooks: function () {
    var oldClass = this.sprite.object.className
    var looks
    
    while (oldClass === this.sprite.object.className) {
      looks = getRandom(combinations)
      this.sprite.object.className = 'player ' + this.team.home_away + ' ' + getRandom(combinations)
    }

    this.setLooks(looks)
  }

, switchMonkey: function() {
    this.monkey = !this.monkey
  }
  
, dribble: function (ball) {
    // Dribbling is 85% similar
    // needs work, it can get better

    if (!this.walks) return

    ball.a = this.a
    if (ball.v < this.speed)
      ball.v = this.speed * 1.06

    var mut = 0.0175

    var ax = (utils.dotLineLength(ball.x, ball.y, this.feet[0].x, this.feet[0].y, this.feet[3].x, this.feet[3].y, false) * 1.30) * mut

    if (ax > 0) ball.v += ax
  }
  
, checkBallOwner: function (ball) {
    var dx = this.cx - ball.x
      , dy = this.cy - ball.y
      , dist = Math.sqrt(dx*dx + dy*dy)
        
    if (dist < 4 + 10 && ball.z < 10) {
      //state.subscribe(['Player', 'Ball'])
    
      return true
     
    } else {
      //state.subscribe(['Player'])
    
      return false
    }
  }
  
, checkBallCollision: function (ball) {
    if (utils.isPointInPoly(this.feet[this.direction], {x: ball.x, y: ball.y})) {
      console.log('collision!')
      return true
    }
    return false
  }

, cpuWalkTo: function(x,y) {
    this.wtx = x
    this.wty = y
  }
  
, cpuWalk: function() {
    var mex = this.x + 12
      , mey = this.y + 30
      , dirx = '', diry = '', totdir = ''
  
    if (mex+8<this.wtx) {
      dirx='right'
    } else if (mex-8>this.wtx) {
      dirx='left'
    }
    if (mey+8<this.wty) {
      diry='down'
    } else if (mey-8>this.wty) {
      diry='up'
    }
    
    if (dirx.length && diry.length) {
      totdir=dirx + ' + ' + diry
    } else if (dirx.length && !diry.length) {
      totdir=dirx
    } else if (!dirx.length && diry.length) {
      totdir=diry
    }

    if (totdir !== '') {
      this.direction = totdir
      this.sprite.animate(totdir)
      if (this.keys == 0) this.walks = false
      else this.walks = true
    } else {
      //this.sprite.stand()
      //this.direction = ''
      //this.walks = false
    }
  }

, generateFeetPlane: function () {
    var mv = 22
      , cx = this.x + 11 + (this.vx * 3.5) + (mv/3)
      , cy = this.y + 29.5 + (this.vy * 4) + (mv/2)
      , x0, y0, x1,y1, x2, y2, x3, y3
      
    /* var h = mv / Math.cosa(this.a)
    */

    x0 = cx
    y0 = cy
    x1 = x0 + (mv*0.54)
    y1 = y0
    x2 = x1
    y2 = y0 - mv
    x3 = x0
    y3 = y2

    this.feet[this.direction] = utils.rotate(this.a, {x: x0 - (mv/3), y: y0 - (mv/2)}, [x0,y0, x1,y1, x2,y2, x3,y3])
    this.feet[this.direction].forEach(function (vec) {
      vec.x -= cx
      vec.y -= cy
    })
    //console.log(this.feet[this.direction])
    //drawB(this.feet)
  }

, makeFeetPlanes: function () {
    for (var k in Direction) {
      if (!k) continue
      this.turnTo(k)
      this.generateFeetPlane()
    }
  }

, turnTo: function (direction) {
    this.direction = direction
    this.a = Direction[this.direction]
    this.sprite.animate(this.direction)
    this.vx = this.speed * Math.cosa(this.a)
    this.vy = this.speed * Math.sina(this.a)
  }

, stand: function () {
    this.sprite.stand()
  }

, jump: function (dir) {
    var self = this
    this.vx = Math.random() * 300
    if (dir === 'left') this.vx = -this.vx
    this.sprite.frame = 0
    this.sprite.duration = this.sprite.map[this.sprite.face][0][1]
    this.jumping = 'jump ' 
                 + (this.home_away === 'home' ? 'up' : 'down')
                 + ' + ' + dir
    this.direction = ''
    setTimeout(function () {
      self.jumping = 0
      self.direction = (self.home_away === 'home' ? 'up' : 'down')
      self.sprite.animate(self.direction)
      self.sprite.stand()
    }, 3000)
  }

, update: function () {
    var game = this.game
    var master = this.game.master
    var ball = this.game.ball
    if (!this.monkey)
      this.handleKeys()
    else {
      if (new Date() - this.monkeyDate > Math.random() * 10000) {
        this.monkeyDate = new Date()
        //this.direction = getRandomProperty(Direction)
        //if (Math.random() * 10000 < 2000) 
        var dx = ball.x - this.x
          , dy = ball.y - this.y
          , dist = Math.sqrt(dx*dx + dy*dy)

        //console.log(dist)
        if (dist < 300 && dist > 10) {
          if (Math.random() * 10000 < 1000) this.cpuWalkTo(ball.x, ball.y)
        }
        
        if ((master.team.home_away === this.team.home_away) && dist > 800) {
          if (Math.random() * 10000 < 5000) this.cpuWalkTo(ball.x, ball.y )
        }        
        
      if (Math.abs(this.wtx-this.x) > 6 && Math.abs(this.wty-this.y) > 6 && master !== this) {
        if (master.team.home_away != this.team.home_away) {
          this.cpuWalkTo(ball.x, ball.y)
          this.cpuWalk()
        }
        if (Math.abs(this.wtx-this.x) > 200 && Math.abs(this.wty-this.y) > 200 && master !== this) {
          this.cpuWalkTo(ball.x, ball.y)
          this.cpuWalk()
          //this.cpuWalkTo(this.x, this.y)
        }
      } else {
        
        //if (master.team.home_away === this.team.home_away) {
          //this.cpuWalkTo(ball.x, ball.y)
        //}
        if (this.keys == 0 && master !== this) this.direction = ''
        //if (master.team.home_away != this.team.home_away && this.keys == 0) this.direction = ''
        
        //this.sprite.stand()
      }

      }
      //console.log(this.wtx, this.x)

    }
    
    this.speed = this.runSpeed

    if (!this.jumping && this.isKeeper && master !== this) {
      if (this.home_away === 'away') {
        this.cpuWalkTo(stadium.post.left + ((ball.x - stadium.post.left + 40) * 0.4), stadium.post.top + (ball.y - stadium.post.top) * 0.5)
      } else {
        this.cpuWalkTo(stadium.post.left + ((ball.x - stadium.post.left + 40) * 0.4), stadium.post.bottom - (stadium.post.bottom - ball.y) * 0.5)
      }
      this.cpuWalk()
    } else if (!this.jumping && game.passTo === this) {
      this.cpuWalkTo(ball.x,ball.y)
      this.cpuWalk()
    }

    if (this.checkBallOwner(ball) && this.kicks===false) {
      if (this.checkBallCollision(ball)) {
        this.speed = this.dribbleSpeed
        this.dribble(ball)
      } 
      
      if (ball.owner !== this && (ball.previousOwner !== this || ball.owner !== null)) {
        ball.v = 0
        ball.vz = 0
        ball.z = 0
        ball.s = 0
        ball.y = this.y+33
        ball.x = this.x+12
        if (master != this && ball.previousOwner == master && ball.owner != master) master.keys = 0
        game.delPassTo()
        if (!this.jumping) this.sprite.stand()
      }

      if (master != this && this.team.home_away === master.team.home_away) {
        var oldClass = master.sprite.object.className
          , oldClassArr = oldClass.split(' ')
        
        if (oldClassArr.indexOf('master') >= 0) oldClassArr.splice(oldClassArr.indexOf('master'),1)
        master.sprite.object.className = oldClassArr.join(' ')
      }

      if (this.team.home_away === master.team.home_away) {
        if (master != this) {      
          master.isMaster = false
          master = this.game.master = this
          master.isMaster = true
          oldClass = master.sprite.object.className
          oldClassArr = oldClass.split(' ')
          oldClassArr.push('master')
          master.sprite.object.className = oldClassArr.join(' ')
        }
        master.monkey = false
      }
      ball.setOwner(this)
      
      game.delPassTo()
    }

    if (this.direction !== '') {
      this.x += this.vx
      this.y += this.vy
    }

    if (this.jumping) {
      this.x += Math.min(3, Math.max(this.vx, -3))
      this.y += Math.min(3, Math.max(this.vy, -3))
      this.vx *= 0.86
      this.vy *= 0.86
    }
    if (this.jumping) this.sprite.animate(this.jumping)    
    
    if (this.x+4 < stadium.bounds.left) this.x = stadium.bounds.left-4
    if (this.x+16 > stadium.bounds.right) this.x = stadium.bounds.right-16
    if (this.y+32 < stadium.bounds.top) this.y = stadium.bounds.top-32
    if (this.y+38 > stadium.bounds.bottom) this.y = stadium.bounds.bottom-38
  }
  
, display: function(a) {  
    this.sprite.object.style.left = (a.x + (this.sprite.offsetX || 0)) + 'px'
    this.sprite.object.style.top = (a.y + (this.sprite.offsetY || 0)) + 'px'
    this.sprite.object.style.zIndex = Math.floor(a.y+30)
  }
}

module.exports = Keeper


});
require.register("./player", function(module, exports, require){
var utils = require('./utils')
var Sprite = require('./sprite')
var Stadium = require('./stadium')
var stadium = new Stadium()

var combinations = [
  'white_black'
, 'white_brown'
, 'white_blonde'
, 'black_black'
, 'black_brown'
, 'black_blonde'
]

// Direction in radius
var Direction = {
  'up': 270
, 'down': 90
, 'left': 180
, 'right': 0
, 'left + up': 225
, 'right + up': 315
, 'left + down': 135
, 'right + down': 45
, 'stand': 0
, '' : 0
}

// Direction by keys
var KeysDirection = {
  0: '' // still
, 1: 'left' //Direction.left
, 2: 'up' //Direction.up
, 3: 'left + up' //Direction.leftUp
, 4: 'right' //Direction.right
, 5: 'left + right' // left + right
, 6: 'right + up' //Direction.rightUp
, 7: '' // impossible
, 8: 'down' //Direction.down
, 9: 'left + down' //Direction.leftDown // left + down
, 10: 'up + down' //null // up + down
, 12: 'right + down' //Direction.rightDown
}

function Player (game, number, uid, home_away) {
  this.game = game

  this.number = number

  this.uid = uid || Math.floor(Math.random() * 100)

  this.home_away = home_away

  this.initDirection = home_away === 'home' ? 'up' : 'down'

  // actual sprite position
  this.x = 0
  this.y = 0

  // walk to xy
  this.wtx = this.x
  this.wty = this.y

  // player feet position
  this.cx = 0
  this.cy = 0

  this.keys = 0

  this.feet = {}

  this.vx = 0
  this.vy = 0

  this.a = 0

  this.ox = this.x
  this.oy = this.y

  this.monkey = false
  this.monkeyDate = new Date()

  this.walks = false

  this.runSpeed = 3.6
  this.dribbleSpeed = 3.4

  this.speed = this.runSpeed

  this.magnet = 0.1
  this.direction = ''
  this.olddirection = ''
  this.kicks = false

  this.kickPower = 12
  this.passPower = 6

  this.jumping = 0
  this.isKeeper = false

  this.m = 8
  /*
  var self = this
  setInterval(function () {
    if (self.isKeeper && !self.jumping) self.jump(utils.getRandom(['right', 'left']))
  }, 5000)*/
  this.team = {}

  this.cpu = false

  return this
}

Player.prototype = {
  createSprite: function () {
    this.sprite = new Sprite(
      'player_' + this.uid
      , 'player ' + this.home_away + ' ' + this.looks
      , [32, 40]
      , { 'up' : [
            [0,10,4,2]
          , [1,10,4,0]
          , [0,10,4,2]
          , [2,10,4,4]
          ]
        , 'down' : [
            [3,10,4,2]
          , [4,10,4,4]
          , [3,10,4,2]
          , [5,10,4,0]
          ]
        , 'right' : [
            [6,10,4,6]
          , [7,10,4,2]
          , [6,10,4,6]
          , [8,10,4,4]
          ]
        , 'left' : [
            [9,10,4,6]
          , [10,10,4,4]
          , [9,10,4,6]
          , [11,10,4,4]
          ]
        , 'left + down' : [
            [13,10,4,8]
          , [12,10,4,6]
          , [13,10,4,8]
          , [14,10,4,4]
          ]
        , 'right + down' : [
            [16,10,4,4]
          , [15,10,4,4]
          , [16,10,4,4]
          , [17,10,4,2]
          ]
        , 'left + up' : [
            [19,10,4,8]
          , [18,10,4,4]
          , [19,10,4,8]
          , [20,10,4,6]
          ]
        , 'right + up' : [
            [22,10,4,4]
          , [21,10,4,2]
          , [22,10,4,4]
          , [23,10,4,4]
          ]
        , 'jump down + right': [
            [24,5,-2,4,2,-4]
          , [25,5,-4,4,2,-4]
          , [26,5,-4,4,2,-4]
          , [27,5,-2,4,4,-4]
          , [28,5,6,4,6,-4]
          , [29,1000,8,4,6,-4]          
          ]
        , 'jump down + left': [
            [36,5,-4,4,4,-4]
          , [35,5,-2,4,2,-4]
          , [34,5,-2,4,2,-4]
          , [33,5,-2,4,4,-4]
          , [32,5,6,4,6,-4]
          , [31,1000,8,4,6,-4]          
          ]
        , 'jump up + right': [
            [38,5,-2,4,2,-4]
          , [39,5,-4,4,2,-4]
          , [40,5,-4,4,2,-4]
          , [41,5,-2,4,4,-4]
          , [42,5,6,4,6,-4]
          , [43,1000,8,4,6,-4]
          ]
        , 'jump up + left': [
            [50,5,-4,4,4,-4]
          , [49,5,-2,4,2,-4]
          , [48,5,-2,4,2,-4]
          , [47,5,-2,4,4,-4]
          , [46,5,6,4,6,-4]
          , [45,1000,8,4,6,-4]
          ]          
        }
      )

    this.makeFeetPlanes()
    this.sprite.face = utils.getRandomProperty(this.sprite.map)
  }

, handleKeys: function () {
    this.olddirection = this.direction

    this.ox = this.x
    this.oy = this.y

    if (this.isMaster) {
      this.keys = keys.getKeys()
      this.turnTo(KeysDirection[this.keys])
      if (this.direction === '') this.walks = false
      else this.walks = true
    }
    else {
      this.turnTo(KeysDirection[this.keys])
      if (this.direction === '') this.walks = false
      else this.walks = true
    }
  }

, setLooks: function (looks) {
    this.looks = looks
  }

, randomLooks: function () {
    this.setLooks(utils.getRandom(combinations))
  }

, switchLooks: function () {
    var oldClass = this.sprite.object.className
    var looks
    
    while (oldClass === this.sprite.object.className) {
      looks = getRandom(combinations)
      this.sprite.object.className = 'player ' + this.team.home_away + ' ' + getRandom(combinations)
    }

    this.setLooks(looks)
  }

, switchMonkey: function() {
    this.monkey = !this.monkey
  }
  
, dribble: function (ball) {
    // Dribbling is 85% similar
    // needs work, it can get better

    if (!this.walks) return

    ball.a = this.a
    if (ball.v < this.speed)
      ball.v = this.speed * 1.06

    var mut = 0.0115
    var ft = this.getFeet()
    var ax = (utils.dotLineLength(ball.x, ball.y, ft[0].x, ft[0].y, ft[3].x, ft[3].y, false) * 1.30) * mut

    if (ax > 0) ball.v += ax
  }
  
, checkBallOwner: function (ball) {
    var dx = this.x - ball.x
      , dy = this.y - ball.y
      , dist = Math.sqrt(dx*dx + dy*dy)
      , minDist
    
    if (this.jumping) minDist = 15
    else minDist = 14
    if (dist < minDist && ( (!this.isKeeper && ball.z < 7) || (this.isKeeper && ball.z < 30) ) ) {
      return true
    } else {
      return false
    }
  }
  
, checkBallCollision: function (ball) {
    if (utils.isPointInPoly(this.getFeet(), { x: ball.x, y: ball.y })) {
      return true
    }
    return false
  }

, cpuWalkTo: function(x,y) {
    this.wtx = x
    this.wty = y
  }
  
, cpuWalk: function() {
    var m = this.m
    var mex = this.x
    var mey = this.y
    var dirx = '', diry = '', totdir = ''
  
    if (mex + m < this.wtx) {
      dirx = 'right'
    } else if (mex-m > this.wtx) {
      dirx = 'left'
    }
    if (mey + m < this.wty) {
      diry = 'down'
    } else if (mey-m>this.wty) {
      diry = 'up'
    }
    
    if (dirx.length && diry.length) {
      totdir = dirx + ' + ' + diry
    } else if (dirx.length && !diry.length) {
      totdir = dirx
    } else if (!dirx.length && diry.length) {
      totdir = diry
    }

    if (totdir !== '') {
      this.turnTo(totdir)
    }
  }

, generateFeetPlane: function () {
    var mv = 26
      , cx = this.x + (this.vx * 3.5) + (mv/3)
      , cy = this.y + (this.vy * 4) + (mv/2)
      , x0, y0, x1,y1, x2, y2, x3, y3
      
    /* var h = mv / Math.cosa(this.a)
    */

    x0 = cx
    y0 = cy
    x1 = x0 + (mv*0.54)
    y1 = y0
    x2 = x1
    y2 = y0 - mv
    x3 = x0
    y3 = y2

    this.feet[this.direction] = utils.rotate(this.a, {x: x0 - (mv/3), y: y0 - (mv/2)}, [x0,y0, x1,y1, x2,y2, x3,y3])
    this.feet[this.direction].forEach(function (vec) {
      vec.x -= cx
      vec.y -= cy
    })

    //console.log(this.feet[this.direction])
    //drawB(this.feet)
  }

, makeFeetPlanes: function () {
    for (var k in Direction) {
      if (!k) k = 'stand'
      this.turnTo(k)
      this.generateFeetPlane()
    }
  }

, getFeet: function () {
    var self = this
    var ret = []
    var feet = this.feet[this.direction] || this.feet['stand']
    var mv = 26
      , cx = this.x + (this.vx * 3.5) + (mv/3)
      , cy = this.y + (this.vy * 4) + (mv/2)    
    feet.forEach(function (vec) {
      ret.push({
        x: cx + vec.x
      , y: cy + vec.y
      })
    })
    //drawB(ret)
    return ret
  }

, turnTo: function (direction) {
    this.direction = direction
    if (this.direction) {
      this.a = Direction[this.direction]
      this.sprite.animate(this.direction)
      this.vx = this.speed * Math.cosa(this.a)
      this.vy = this.speed * Math.sina(this.a)
      this.walks = true
    } else if (!this.jumping) {
      this.vx = 0
      this.vy = 0
      this.sprite.stand()
      this.walks = false
    }
  }

, stand: function () {
    this.vx = 0
    this.vy = 0
    this.sprite.stand()
  }

, jump: function () {
    var self = this
    var diff = (this.game.ball.x - this.x) + this.game.ball.vx * 10
    this.vx = diff
    //if (this.initDirection === 'up') this.vx = -this.vx
    var dir = diff >= 0 ? 'right' : 'left'
    //this.sprite.duration = this.sprite.map[this.sprite.face][0][1]
    this.direction = dir
    this.a = Direction[dir]
    this.vy = 0
    this.jumping = 'jump '
                 + this.initDirection
                 + ' + ' + dir
    this.sprite.frame = 0
    this.sprite.animate(this.jumping)
    setTimeout(function () {
      self.jumping = 0
      self.direction = self.initDirection
      self.a = Direction[self.direction]
      self.sprite.animate(self.direction)
      self.sprite.stand()
      if (self.game.ball.owner === self) {
        setTimeout(function () {
          if (self.game.ball.owner === self) {
            self.game.ball.shoot()
          }
        }, 500)
      }
    }, 2000)
  }

, update: function () {
    var game = this.game
    var master = this.game.master
    var ball = this.game.ball
    if (!this.monkey) {
      if (this.isKeeper && ball.owner !== this) {
      }
      else {
        if (!this.jumping) this.handleKeys()
      }
    }
    else {
      if (new Date() - this.monkeyDate > Math.random() * 10000) {
        this.monkeyDate = new Date()
        //this.direction = getRandomProperty(Direction)
        //if (Math.random() * 10000 < 2000) 
        var dx = ball.x - this.x
          , dy = ball.y - this.y
          , dist = Math.sqrt(dx*dx + dy*dy)

        //console.log(dist)
        if (dist < 300 && dist > 10) {
          if (Math.random() * 10000 < 1000) this.cpuWalkTo(ball.x, ball.y)
        }
        
        if ((master.team.home_away === this.team.home_away) && dist > 800) {
          if (Math.random() * 10000 < 5000) this.cpuWalkTo(ball.x, ball.y )
        }        
        
      if (Math.abs(this.wtx-this.x) > 6 && Math.abs(this.wty-this.y) > 6 && master !== this) {
        if (master.team.home_away != this.team.home_away) {
          this.cpuWalkTo(ball.x, ball.y)
          this.cpuWalk()
        }
        if (Math.abs(this.wtx-this.x) > 200 && Math.abs(this.wty-this.y) > 200 && master !== this) {
          this.cpuWalkTo(ball.x, ball.y)
          this.cpuWalk()
          //this.cpuWalkTo(this.x, this.y)
        }
      } else {
        
        //if (master.team.home_away === this.team.home_away) {
          //this.cpuWalkTo(ball.x, ball.y)
        //}
        if (this.keys == 0 && master !== this) this.direction = ''
        //if (master.team.home_away != this.team.home_away && this.keys == 0) this.direction = ''
        
        //this.sprite.stand()
      }

      }
      //console.log(this.wtx, this.x)

    }
    
    this.speed = this.runSpeed

    if (!this.jumping && this.isKeeper && master !== this) {
      var tx, ty
      if (this.home_away === 'away') {
        if (ball.y < stadium.keeper.top.bottom
          && ball.x > stadium.keeper.left && ball.x < stadium.keeper.right
          && ball.y > stadium.field.top - 60) {
          tx = ball.x
          ty = ball.y
          if (ball.z > 3 && !this.jumping && !this.kicks && ball.y - this.y < 30) this.jump(this.vx >= 0 ? 'right' : 'left')
        }
        else {
          tx = stadium.post.left + (ball.x - stadium.post.left + 60) * 0.4
          ty = stadium.post.top + (ball.y - stadium.post.top) * 0.3
        }
        if (!this.jumping && ball.owner !== this) this.cpuWalkTo(tx, ty)
      } else {
        if (ball.y > stadium.keeper.bottom.top
          && ball.x > stadium.keeper.left && ball.x < stadium.keeper.right
          && ball.y < stadium.field.bottom + 60) {
          tx = ball.x
          ty = ball.y
          if (ball.z > 3 && !this.jumping && !this.kicks && this.y - ball.y < 30) this.jump(this.vx >= 0 ? 'right' : 'left')
        }
        else {
          tx = stadium.post.left + (ball.x - stadium.post.left + 60) * 0.4
          ty = stadium.post.bottom - (stadium.post.bottom - ball.y) * 0.3
        }
        if (!this.jumping && ball.owner !== this) this.cpuWalkTo(tx, ty)
      }
      if (!this.jumping && ball.owner !== this) this.cpuWalk()
    } else if (!this.jumping && game.passTo === this) {
      this.cpuWalkTo(ball.x,ball.y)
      this.cpuWalk()
    }

    if (this.checkBallOwner(ball) && this.kicks===false) {
      //console.log('ball owner!')
      if (this.checkBallCollision(ball)) {
        this.speed = this.dribbleSpeed
        if (!this.isKeeper) this.dribble(ball)
      } 
      
      if (!this.kicks && ball.owner !== this && ball.previousOwner !== this) {
        ball.x = this.x
        ball.y = this.y
        ball.v = 0
        ball.vz = 0
        ball.z = 0
        ball.s = 0
        if (master != this && ball.previousOwner == master && ball.owner != master) master.keys = 0
        game.delPassTo()
        //if (!this.jumping) this.sprite.stand()
      }

      if (master != this && this.team.home_away === master.team.home_away) {
        var oldClass = master.sprite.object.className
          , oldClassArr = oldClass.split(' ')
        
        if (oldClassArr.indexOf('master') >= 0) oldClassArr.splice(oldClassArr.indexOf('master'),1)
        master.sprite.object.className = oldClassArr.join(' ')
      }

      if (this.team.home_away === master.team.home_away) {
        if (master != this) {      
          master.isMaster = false
          //master.monkey = true
          master = this.game.master = this
          master.isMaster = true
          //master.monkey = false
          oldClass = master.sprite.object.className
          oldClassArr = oldClass.split(' ')
          oldClassArr.push('master')
          master.sprite.object.className = oldClassArr.join(' ')
        }
        master.monkey = false
      }
      ball.setOwner(this)
      
      game.delPassTo()
    }

    if (this.direction !== '' && !this.jumping) {
      this.x += this.vx
      this.y += this.vy
    } else if (this.jumping) {
      this.x += Math.min(4, Math.max(this.vx, -4))
      this.y += Math.min(4, Math.max(this.vy, -4))
      this.vx *= 0.86
      this.vy *= 0.86
      if (ball.owner === this) {
        ball.x = this.x
        ball.y = this.y
        if (ball.previousOwner !== this) {
          ball.vz = 1
        }
      }
      this.sprite.animate(this.jumping)
    } else {
      this.stand()
    }

    if (this.x < stadium.bounds.left) this.x = stadium.bounds.left
    if (this.x > stadium.bounds.right) this.x = stadium.bounds.right
    if (this.y < stadium.bounds.top) this.y = stadium.bounds.top
    if (this.y > stadium.bounds.bottom) this.y = stadium.bounds.bottom
  }
  
, display: function(a) {  
    this.sprite.object.style.left = (a.x + (this.sprite.offsetX || 0)) + 'px'
    this.sprite.object.style.top = (a.y + (this.sprite.offsetY || 0)) + 'px'
    this.sprite.object.style.zIndex = Math.floor(a.y)
  }
}

module.exports = Player


});
require.register("./sprite", function(module, exports, require){
function Sprite (sid,clas,size,map) {
  var obj = this.object = document.createElement('div')
  obj.setAttribute('id',sid)
  obj.className = clas
  obj.style.position = 'absolute'
  obj.style.width = size[0] + 'px'
  obj.style.height = size[1] + 'px'
  obj.style.zIndex = 1
  document.body.appendChild(obj)

  this.map = map
  this.size = size
  this.face = null
  this.frame = 0
  this.duration = 1
  this.current = []
}

Sprite.prototype = {  
  animate: function (face) {
    if (this.face !== face) {
      this.face = face
    }
    if (typeof this.map[this.face] == 'undefined' || typeof this.map[this.face][this.frame] == 'undefined') return
    this.object.style.backgroundPosition
    = [ this.map[this.face][this.frame][3]
      , 'px '
      , - ( this.map[this.face][this.frame][0] * this.size[1]
          - this.map[this.face][this.frame][2]
          )
      , 'px'
      ].join('')
    if (this.duration <= 1) {
      this.frame++
      if (this.frame >= this.map[this.face].length) this.frame=0
      this.duration = this.map[this.face][this.frame][1]
    } else {
      this.duration--
    }
    this.offsetX = this.map[this.face][this.frame][5]
    this.offsetY = this.map[this.face][this.frame][4]
  }
  
, stand: function() {
    this.frame = 0
    this.duration = 1
    this.animate(this.face)
  }
}

module.exports = Sprite

});
require.register("./ball", function(module, exports, require){
/*
 *
 *  Ball
 *
 */
 
var Stadium = require('./stadium')
var stadium = new Stadium

// Direction in radius
var Direction = {
  'up': 270
, 'down': 90
, 'left': 180
, 'right': 0
, 'left + up': 225
, 'right + up': 315
, 'left + down': 135
, 'right + down': 45
, '' : null
}

// Direction by keys
var KeysDirection = {
  0: '' // still
, 1: 'left' //Direction.left
, 2: 'up' //Direction.up
, 3: 'left + up' //Direction.leftUp
, 4: 'right' //Direction.right
, 5: 'left + right' // left + right
, 6: 'right + up' //Direction.rightUp
, 7: '' // impossible
, 8: 'down' //Direction.down
, 9: 'left + down' //Direction.leftDown // left + down
, 10: 'up + down' //null // up + down
, 12: 'right + down' //Direction.rightDown
}

function Ball (game, props) {
  this.game = game

  // get ball and shadow dom elements
  this.ball = document.getElementById('ball')
  this.shadow = document.getElementById('ball_shadow')

  this.g = 0.088 // gravity
  this.f = 0.966 // ground friction
  this.af = 0.974 // air friction
  
  this.a = 0 // angle
  this.s = 0 // spin
  
  // xyz coords
  this.x = ((stadium.bounds.right - stadium.bounds.left) / 2) + stadium.bounds.left
  this.y = ((stadium.bounds.bottom - stadium.bounds.top) / 2) + stadium.bounds.top
  this.z = 0

  this.v = 0 // xy velocity
  this.vz = 0 // z velocity
  
  // xy velocity splitted
  this.vx = 0
  this.vy = 0

  // is ball in nets
  this.nets = false
  
  // who's control of the ball
  this.owner = {
    current: null
  , previous: null
  }
  
  // who kicked the ball and where
  this.kick = {
    owner: null
  , direction: null
  }

  // property override
  for (var p in props) {
    this[p] = props[p]
  }

  return this  
}

Ball.prototype = {
  // examine collision with boundaries and physical objects
  hitBounds: function() {
    // bounce determining helper boolean
    var bounce = false
    
    //
    // stadium boundaries bounce
    //
    
    if (this.x - 4 < stadium.bounds.left)
      this.vx = -this.vx, this.x = stadium.bounds.left + 4, bounce=true
    else if (this.x+4 > stadium.bounds.right)
      this.vx = -this.vx, this.x = stadium.bounds.right - 4, bounce=true
      
    if (this.y-4 < stadium.bounds.top)
      this.vy = -this.vy, this.y = stadium.bounds.top + 4, bounce=true
    else if (this.y+4 > stadium.bounds.bottom)
      this.vy = -this.vy, this.y = stadium.bounds.bottom - 4, bounce=true
    
    // if we bounced, calculate new angle and dampen velocity
    if (bounce) {
      this.a = Math.atan2(this.vy, this.vx) * 180 / Math.PI
      this.v *= 0.8
    }

    // reset bounce helper
    bounce = false
    
    // is the ball in nets area?
    if (this.y-4 < stadium.post.top && this.y+4 > stadium.post.top - stadium.post.depthTop
     && this.x-4 > stadium.post.left && this.x+4 < stadium.post.right
     && this.z-4 < stadium.post.z) {
      this.nets = true
    }

    if (this.y+4 > stadium.post.bottom && this.y-4 < stadium.post.bottom + stadium.post.depthBottom
     && this.x-4 > stadium.post.left && this.x+4 < stadium.post.right
     && this.z-4 < stadium.post.z) {
      this.nets = true
    }

    //
    // posts bounce
    //


    // iterate through posts and do simple circle to circle collision
  
    var post, dx, dy, dist, angle, rangle, sx, sy
      , minDist = 7
    
    for (var i=stadium.posts.vert.length; i--;) {
      post = stadium.posts.vert[i]
      
      dx = post.x - this.x
      dy = post.y - this.y
      dist = Math.sqrt(dx*dx + dy*dy)

      if (dist < minDist) {
        angle = Math.atan2(dy, dx) * 180 / Math.PI
        this.x += Math.cos(angle) * (minDist - dist)
        this.y += Math.sin(angle) * (minDist - dist)
        sx = this.x - post.x
        sy = this.y - post.y
        
        this.a = Math.atan2(sy, sx) * 180 / Math.PI
      }
    }
    
    for (var i=stadium.posts.horiz.length; i--;) {
      post = stadium.posts.horiz[i]
      
      dz = post.z - this.z
      dy = post.y - this.y
      dist = Math.sqrt(dz*dz + dy*dy)

      if (dist < minDist) {
        angle = Math.atan2(dy, dz) * 180 / Math.PI
        this.z += Math.cos(angle) * (minDist - dist)
        this.y += Math.sin(angle) * (minDist - dist)
        sz = this.z - post.z
        sy = this.y - post.y
        
        rangle = Math.atan2(sy, sz) * 180 / Math.PI
        
        this.vz = this.vx * Math.cosa(rangle)
        this.vy = this.v * Math.sina(rangle)
        
        this.v = Math.sqrt(this.vx*this.vx + this.vy*this.vy)
        
        this.a = Math.atan2(this.vy, this.vx) * 180 / Math.PI
      }
    }

    // are we in the nets area at all?
    if (this.nets) {

      //
      // nets bounce and keeping it inside
      //

      // keep the ball in nets and calculate net collision behaviour
      if (this.x - 4 < stadium.post.left)
        this.vx = -this.vx * 0.02, this.vy *= 1.21, this.x = stadium.post.left + 4, bounce=true
      else if (this.x + 4 > stadium.post.right)
        this.vx = -this.vx * 0.02, this.vy *= 1.21, this.x = stadium.post.right - 4, bounce=true
        
      if (this.y - 4 < stadium.post.top - stadium.post.depthTop)
        this.vy = -this.vy * 0.02, this.vx *= 1.21, this.y = stadium.post.top - stadium.post.depthTop + 4, bounce=true

      if (this.y + 4 > stadium.post.bottom + stadium.post.depthBottom)
        this.vy = -this.vy * 0.02, this.vx *= 1.21, this.y = stadium.post.bottom + stadium.post.depthBottom - 4, bounce=true
      
      // if ball hits the nets ceiling, put it back in, calculate nets curve and kill z velocity
      if (this.y - 4 < stadium.post.top) {
        if (this.z-4 > stadium.post.z - ((stadium.post.top - this.y) * 0.65)) {
          this.z = stadium.post.z - ((stadium.post.top - this.y) * 0.65)
          this.vz = 0
        }
      }

      if (this.y + 4 > stadium.post.bottom) {
        if (this.z-4 > stadium.post.z - ((stadium.post.bottom - this.y) * 0.65)) {
          this.z = stadium.post.z - ((stadium.post.bottom - this.y) * 0.65)
          this.vz = 0
        }
      }
      
      // if we bounced calculate new angle, and dampen velocities
      if (bounce) {
        this.a = Math.atan2(this.vy, this.vx) * 180 / Math.PI
        this.v = Math.sqrt(this.vx*this.vx + this.vy*this.vy) * 0.85
        if (this.vz > 0) this.vz *= 0.5
      }
    }
    
    // not in nets any more
    if (this.y - 4 > stadium.post.top && this.y + 4 < stadium.post.bottom) this.nets = false

  }
  
, afterTouch: function() {
    // is it kicked and is user swerving?
    if (this.kick.owner !== null && this.kick.direction !== null && this.kick.owner.walks) {
      //state.subscribe(['Player'])   
      
      // calculate difference in degrees
      var dif = ((this.a - Direction[this.kick.owner.direction] + 540) % 360) - 180
        , adif = Math.abs(dif)
        , l = 0
        , s = 0
      
      // get swerve and lob by degree difference
      if (adif >= 0 && adif < 90) {
        s = 1.3
        l = 0.10
        this.v += (this.v) * 0.0029
      } else if (adif >= 90 && adif < 135) {
        s = 0.9
        l = 0.65
      } else if (adif >= 135) {
        s = l = 1
      }

      // apply lob
      this.vz += this.v * l * 0.0236

      // apply swerve
      if (adif > 0 && adif < 180)
        this.s += this.v * 0.0092 * s * -Math.sgn(dif)
    }
  }
  
, pass: function() {
    var self = this
    
    // set kick owner and copy owner properties
    this.kick.owner = this.owner
    //this.kick.direction = this.kick.owner.direction
    
    if (this.kick.direction !== '') {
      // this prevents ball from being owned immediately after shooting
      this.kick.owner.kicks = true
      setTimeout(function() {
        self.kick.owner.kicks = false
        //state.subscribe(['Player'])   
        
      }, 200)
    
      // get closest player to where the kicker faces
      var passTo = this.kick.owner.team.getClosest(
          this.x + (300 * Math.cosa(this.a))
        , this.y + (300 * Math.sina(this.a))
        , this.kick.owner)
      
      // calculate ball direction
      var dx = passTo.x - this.x
        , dy = passTo.y - this.y
        , dist = Math.sqrt(dx*dx + dy*dy)
        , angle = Math.atan2(dy, dx) * 180 / Math.PI
   
      this.a = angle
      
      // apply forces
      this.v = Math.max(dist * 0.0125, 8)
      this.vz = dist * 0.002
      
      // tell the game where we're passing to
      this.game.setPassTo(passTo)

      // this will stop swerve and lob calculating after 300ms
      setTimeout(function() {
        if (!self.kick.owner.kicks)
          self.kick.direction = null
      }, 300)
      
      // nobody's owner of the ball now that we kicked it
      this.setOwner(null)
    }
  }
  
, shoot: function () {
    var self = this
    console.log('KICK OWNER', this.owner)
    this.kick.owner = this.owner

    //if (this.kick.owner.direction !== '') {
      // get shoot properties from owner direction and set owner as kicker
      this.kick.direction = this.kick.owner.direction
      
      // this prevents ball from being owned immediately after shooting
      this.kick.owner.kicks = true
      setTimeout(function() {
        self.kick.owner.kicks = false
        //state.subscribe(['Player'])
      }, 200)
      
      // reset swerve, set new angle and apply forces
      this.s = 0
      this.a = this.kick.owner.a
      this.v = this.kick.owner.kickPower
      this.vz = this.kick.owner.kickPower*0.12

      // get closest player to where the kicker faces
      var passTo = this.kick.owner.team.getClosest(
          this.x + (800 * Math.cosa(this.a))
        , this.y + (800 * Math.sina(this.a))
        , this.kick.owner)
      
      // tell the game where we're passing to
      this.game.setPassTo(passTo)
      
      // stop calculating aftertouch after 500ms
      setTimeout(function() {
        self.kick.direction = null
        //state.subscribe(['Player'])
      }, 300)
      
      // nobody's owner now
      this.setOwner(null)       
    //}

  }
  
, setOwner: function(obj) {
    // if the ball isn't kicked (obj=null) store the previous owner so he doesn't get it back immediately
    if (obj !== null) {
      this.previousOwner = this.owner
      //state.subscribe(['Player', 'Ball'])
    } else {
      //state.subscribe(['Player'])
    }

    // set the new owner
    this.owner = obj
  }
  
, update: function() {
    //var pos = this.buffer.shift()

    this.a += this.s // apply swerve to angle
    this.s *= 0.97 // dampen swerve force
    this.afterTouch() // calculate aftertouch
    this.vz -= this.g // apply gravity    
    this.z += this.vz // apply z force

    // bounce on grass
    if (this.z < 0) {
      this.vz = -this.vz * 0.72 // reverse and dampen z velocity      
      this.z = 0 // make sure we aren't below ground
      
      // this prevents further aftertouch on the ball
      this.kick.direction = null
    }
    
    this.v += this.z * 0.0011 // transfer a bit of z to xy velocity

    // calculate new xy velocities
    this.vx = this.v * Math.cosa(this.a)
    this.vy = this.v * Math.sina(this.a)
    
    // apply xy velocities
    this.x += this.vx
    this.y += this.vy

    // dampen velocity by friction or air friction
    this.v *= (this.z<=0.1) ? this.f : this.af

    // check if ball hits any bounds or physical objects
    this.hitBounds()
  }

, display: function (a) {
    // change ball position
    this.ball.style.left = a.x + 'px'
    this.ball.style.top = (a.y - a.z) + 'px'
    this.ball.style.zIndex = Math.floor(a.y + 2)

    // change shadow position
    this.shadow.style.left = (a.x + a.z + 2) + 'px'
    this.shadow.style.top = (a.y + (a.z/4) + 2) + 'px'
  }
}

module.exports = Ball

});
require.register("./game", function(module, exports, require){
var Camera = require('./camera')
var State = require('./state')
var Ball = require('./ball')
var Team = require('./team')

function Game () {
  this.master = null
  this.objects = []
  this.passTo = {}
  this.players = {}
  this.player_uid = 0
}

Game.prototype = {
  init: function (opts) {
    var self = this
    this.setFPS(opts.fps)
    this.state = new State(this)
    this.camera = new Camera()
    this.ball = new Ball(this, opts.ballProperties)
    var teams = this.teams = {
      home: new Team(this, 'home')
    , away: new Team(this, 'away')
    }

    this.master = teams.home.createPlayer()
    this.master.isMaster = true
    this.master.randomLooks()
    this.master.createSprite()
    this.addPlayer(this.master)
    this.addObject(this.master)

    var player
    var keeper

    ;(function () {
      keeper = teams.home.createPlayer()
      keeper.isKeeper = true
      keeper.m = 2
      keeper.runSpeed = 3
      keeper.randomLooks()
      keeper.createSprite()
      keeper.sprite.object.classList.add('keeper')
      self.addPlayer(keeper)
      self.addObject(keeper)
    }())

    ;(function () {
      keeper = teams.away.createPlayer()
      keeper.isKeeper = true
      keeper.m = 2
      keeper.runSpeed = 3
      keeper.randomLooks()
      keeper.createSprite()
      keeper.sprite.object.classList.add('keeper')
      self.addPlayer(keeper)
      self.addObject(keeper)      
    }())

    for (var i = 9; i--;) {
      ;(function () {
        var player = teams.home.createPlayer()
        player.randomLooks()
        player.createSprite()
        self.addPlayer(player)
        self.addObject(player)
      }())
    }
    for (var i = 10; i--;) {
      ;(function () {      
        player = teams.away.createPlayer()
        player.randomLooks()
        player.createSprite()
        self.addPlayer(player)
        self.addObject(player)
      }())
    }

    this.addObject(this.ball)
    this.addObject(this.camera)
  }

, setFPS: function (fps) {
    this.fps = fps
    this.ms = Math.floor(1000 / fps)
  }
, addPlayer: function (player) {
    this.players[player.uid] = player
  }
, addObject: function(obj) {
    this.objects.push(obj)
    this.state.watch(obj)
  }

, setPassTo: function(playerobj) {
    this.passTo = playerobj
  }

, delPassTo: function() {
    this.passTo.walks = false
    this.passTo.direction = ''
    this.passTo = {}
  }

, update: function() {
    for (var i=0, l=this.objects.length; i<l; i++) {
      this.objects[i].update()
    }
  }

, loop: function () {
    var self = this, cx, cy, vert, horiz
    ;(function loop () {
      self.state.tick(self.ms)
      horiz = Math.sgn(self.vx)
      vert = Math.sgn(self.vy)
      cx = self.master.x/2 + self.ball.x/2
      cy = self.master.y/2 + self.ball.y/2 + (self.ball.vy * 5) + (Math.sgn(self.ball.vy) * self.ball.z)
      if ((self.ball.x < cx - self.camera.windowWidth/2) || (self.ball.x > (cx - self.camera.windowWidth/2) + self.camera.windowWidth)) {
        cx = self.ball.x + (horiz * 10) + self.camera.windowWidth/2
      } else {
        cx = self.ball.x + (horiz * 10)
      }
      if (self.ball.y < cy - self.camera.windowHeight/2 || self.ball.y > (cy - self.camera.windowHeight/2) + self.camera.windowHeight) {
        cy = self.ball.y + (vert * 10) + (vert * self.ball.z * 3)
      } else {
        cy = self.ball.y + (vert * 10) + (vert * self.ball.z * 3)
      }      
      self.camera.moveTo({
        x: cx
      , y: cy
      })
      window.requestAnimFrame(loop)
    }());
  }
}

module.exports = Game

window.requestAnimFrame = (function () {
  return window.requestAnimationFrame  
      || window.webkitRequestAnimationFrame 
      || window.mozRequestAnimationFrame    
      || window.oRequestAnimationFrame      
      || window.msRequestAnimationFrame     
      || function (callback, el) {
        return window.setTimeout(callback, 1000 / 60)
      }
})();

});
require.register("./team", function(module, exports, require){
var utils = require('./utils')
var Player = require('./player')
var Keeper = require('./keeper')
var Stadium = require('./stadium')
var stadium = new Stadium()

var tactics = {
  '442': [
    [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,11,0,0,0,0,0,9,0,0,0,0 ]
  , [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,6,0,0,0,0,0,0,0,0,0,0,0,7,0  ]
  , [ 0,0,0,0,0,0,0,8,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,10,0,0,0,0,0,0,0 ]
  , [ 0,0,3,0,0,0,0,0,0,0,0,0,2,0,0  ]
  , [ 0,0,0,0,0,5,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,0,0,4,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ]
  , [ 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0  ]
  ]
}

function Team (game, home_away) {
  this.game = game
  this.home_away = home_away || utils.getRandom(['home', 'away'])
  this.lineup = []
  this.player_number = 0
}

Team.prototype = {
  createKeeper: function () {
    var id = this.home_away + '1'
    var keeper = new Keeper(this.game, 1, id, this.home_away)
    this.resetPos(keeper, tactics['442'])
    keeper.team = this
    this.lineup.push(keeper)
    return keeper
  }

, createPlayer: function (id) {
    if (!id) id = this.home_away + (++this.player_number)
    var player = new Player(this.game, this.player_number, id, this.home_away)
    this.resetPos(player, tactics['442'])
    player.team = this
    this.lineup.push(player)
    return player
/*
    if (typeof game.players[id] === 'undefined') {
      game.players[id] = new Player(id, this.home_away)
      game.addObject(game.players[id])
      this.lineup.push(game.players[id])
      game.players[id].team = this
      if (ismaster) master = game.players[id]
      var oldClass = master.sprite.object.className
        , oldClassArr = oldClass.split(' ')
      
      oldClassArr.push('master')
      master.sprite.object.className = oldClassArr.join(' ')
    }*/
  }

, resetPos: function (player, f) {
    var mx = 62
    var my = 74
    f.forEach(function (l, y) {
      l.forEach(function (n, x) {
        if (n == player.number) {
          if (player.home_away === 'home') {
            player.x = ((x + 1) * mx) + stadium.field.left
            player.y = ((y + 1) * my) + stadium.field.top
          } else {
            player.x = ((14 - x + 1) * mx) + stadium.field.left
            player.y = ((15 - y + 1) * my) + stadium.field.top
          }
        }
      })
    })
  }

, hasPlayer: function (player) {
    return !!~this.lineup.indexOf(player)
  }

, getClosest: function(x,y,excl) {
    var dx,dy,di,mindi=100000,co
    for (var i=0, l=this.lineup.length; i<l; i++) {
      if (this.lineup[i] !== excl) {
        dx = this.lineup[i].x
        dy = this.lineup[i].y
        di = Math.abs(Math.sqrt(Math.pow(x - dx,2) + Math.pow(y - dy,2)))
        if (di<mindi) {
          mindi=di
          co=this.lineup[i]
        }
      }
    }
    return co
  }
}

module.exports = Team

});
require.register("./client-app", function(module, exports, require){
/*
 *
 *  nodesoccer
 *
 *  (c) 2010-2012 stagas
 *  All rights reserved
 *
 */

var options = {
  fps: 60
, ballProperties: {
    g: 0.087
  , f: 0.988
  , af: 0.981
  }
}

window.keys = new(require('./keys'))

window.onload = function () {
  var $helper = document.getElementById('helper')  
  var $console = document.getElementById('console')
  var ballbegpos = [650.9,1180]
  var playerbegpos = [650.9,1150]

  var stadium = new(require('./stadium'))
  stadium.init()

  var game = new(require('./game'))
  game.init(options)
  console.log(game)
  keys.init(game)

  game.master.x = game.ball.x - 5
  game.master.y = game.ball.y - 50
  game.master.monkey = false

  setTimeout(function () {
    game.loop()
  }, 100)

  var canvas = exports.canvas = document.getElementById('c')

  window.draw = function draw(x1,y1, x2,y2) {
    var ctx = canvas.getContext('2d')
    ctx.globalCompositeOperation = 'destination-over'
    ctx.clearRect(0,0,1000,1000)
    ctx.beginPath()
    ctx.moveTo(x1,y1)
    ctx.lineTo(x2,y2)
    ctx.stroke()
  }  

  window.drawB = function drawB(ar) {
    var ctx = canvas.getContext('2d')
    ctx.globalCompositeOperation = 'destination-over'
    //ctx.clearRect(0,0,2000,2000)
    
    ctx.fillStyle = "rgba(255,0,0,0.4)"
    ctx.beginPath()
    ctx.moveTo(ar[0].x, ar[0].y)
    for (var i=0; i<ar.length; i++) {
      ctx.lineTo(ar[i].x,ar[i].y)
    }
    ctx.stroke()
    ctx.fill()
  }  

} // onload


});
;(function () {
  console.log('requiring client app')
  require('./client-app')
})();